@@ -538,16 +538,23 @@ class SpiceViewer(Viewer):
         GObject.GObject.connect(self._spice_session, "channel-new",
                                 self._channel_new_cb)
 
-        self._usbdev_manager = SpiceClientGLib.UsbDeviceManager.get(
-                                    self._spice_session)
-        self._usbdev_manager.connect("auto-connect-failed",
-                                    self._usbdev_redirect_error)
-        self._usbdev_manager.connect("device-error",
-                                    self._usbdev_redirect_error)
+        # Distros might have usb redirection compiled out, like OpenBSD
+        # https://bugzilla.redhat.com/show_bug.cgi?id=1348479
+        try:
+            self._usbdev_manager = SpiceClientGLib.UsbDeviceManager.get(
+                                        self._spice_session)
+            self._usbdev_manager.connect("auto-connect-failed",
+                                        self._usbdev_redirect_error)
+            self._usbdev_manager.connect("device-error",
+                                        self._usbdev_redirect_error)
 
-        autoredir = self.config.get_auto_redirection()
-        if autoredir:
-            gtk_session.set_property("auto-usbredir", True)
+            autoredir = self.config.get_auto_redirection()
+            if autoredir:
+                gtk_session.set_property("auto-usbredir", True)
+        except:
+            self._usbdev_manager = None
+            logging.debug("Error initializing spice usb device manager",
+                exc_info=True)
 
 
     #####################
@@ -555,6 +562,8 @@ class SpiceViewer(Viewer):
     #####################
 
     def _main_channel_event_cb(self, channel, event):
+        self._tunnels.unlock()
+
         if event == SpiceClientGLib.ChannelEvent.CLOSED:
             self._emit_disconnected()
         elif event == SpiceClientGLib.ChannelEvent.ERROR_AUTH:
@@ -607,7 +616,6 @@ class SpiceViewer(Viewer):
 
         if (type(channel) == SpiceClientGLib.MainChannel and
             not self._main_channel):
-            self._tunnels.unlock()
             self._main_channel = channel
             hid = self._main_channel.connect_after("channel-event",
                 self._main_channel_event_cb)
